module generic_state_space
  use mkl95_precision, only: wp => dp

  use prior
  use filter

  implicit none

  
  character(len=*), parameter :: mname = 'gss'

  character(len=*), parameter :: priorfile = 'models/gss/prior.txt'
  character(len=*), parameter :: datafile = 'models/gss/CRM.txt'
  character(len=*), parameter :: transfile = 'models/gss/trspec.txt'
  character(len=*), parameter :: varfile = 'models/gss/crmVj.txt'

  integer, parameter :: npara = 60, ny = 10, nobs = 200, ns = 5, neps = 5
  integer, parameter :: t0 = 0

  real(wp), parameter :: REALLY_NEG = -10.0_wp**10

  ! data stuff 
  real(wp) :: YY(ny,nobs)

  ! prior stuff
  integer :: pshape(npara), pmask(npara)
  real(wp) :: pmean(npara), pstdd(npara), pfix(npara)

  integer :: pshape2(npara), pmask2(npara)
  real(wp) :: pmean2(npara), pstdd2(npara), pfix2(npara)


  ! prtr
  real(wp) :: trspec(4, nobs)
contains

  include '../../base/helper_functions.f90'

  subroutine sysmat(para,TT,RR,QQ,DD,ZZ,HH,info)
    ! Populates the system matrices...

    real(wp), intent(in) :: para(60)
    integer, intent(out) :: info
    real(wp), intent(out) :: TT(5,5), RR(5,5), QQ(5,5), DD(10), ZZ(10,5), HH(10,10)

    integer :: i, j, pwedge

    TT = 0.0_wp
    RR = 0.0_wp
    QQ = 0.0_wp
    ZZ = 0.0_wp
    HH = 0.0_wp

    pwedge = 8
    j = 6

    do i = 1,5
       ! state transition equation
       TT(i,i) = para(i)
       RR(i,i) = 1.0_wp
       QQ(i,i) = 1.0_wp

       ! identification restritions on the state-loading matrix ZZ
       ZZ(i,i) = 1.0_wp
       ZZ(i+1:10,i) = para(j:j+pwedge)

       j = j + pwedge + 1
       pwedge = pwedge - 1

    end do

    ! measurement equation
    DD = para(41:50)
    do i = 1,10
       HH(i,i) = exp(para(50+i))
    end do

    info = 0
  end subroutine sysmat

  subroutine sysmat_diff(para,psel,neffpara,DTT,DRR,DQQ,DDD,DZZ,DHH)

    integer, intent(in) :: neffpara, psel(neffpara)
    real(wp), intent(in) :: para(npara)


    real(wp), intent(out) :: DTT(25,neffpara), DRR(25,neffpara), DQQ(25,neffpara)
    real(wp), intent(out) :: DZZ(50,neffpara), DDD(10,neffpara), DHH(100,neffpara)

    real(wp) :: DTTbig(25,npara), DRRbig(25,npara), DQQbig(25,npara)
    real(wp) :: DZZbig(50,npara), DDDbig(10,npara), DHHbig(100,npara)

    integer :: i

    

    ! DTT
    DTTbig = 0.0_wp
    DTTbig(1,1)  = 1.0_wp 
    DTTbig(7,2)  = 1.0_wp 
    DTTbig(13,3) = 1.0_wp 
    DTTbig(19,4) = 1.0_wp 
    DTTbig(25,5) = 1.0_wp 


    ! DRR
    DRRbig = 0.0_wp

    ! DQQ
    DQQbig = 0.0_wp

    ! DZZ 
    DZZbig = 0.0_wp
    DZZbig(2,6)   = 1.0_wp
    DZZbig(3,7)   = 1.0_wp
    DZZbig(4,8)   = 1.0_wp
    DZZbig(5,9)   = 1.0_wp
    DZZbig(6,10)  = 1.0_wp
    DZZbig(7,11)  = 1.0_wp
    DZZbig(8,12)  = 1.0_wp
    DZZbig(9,13)  = 1.0_wp
    DZZbig(10,14) = 1.0_wp
                  
    DZZbig(13,15) = 1.0_wp
    DZZbig(14,16) = 1.0_wp
    DZZbig(15,17) = 1.0_wp
    DZZbig(16,18) = 1.0_wp
    DZZbig(17,19) = 1.0_wp
    DZZbig(18,20) = 1.0_wp
    DZZbig(19,21) = 1.0_wp
    DZZbig(20,22) = 1.0_wp
                  
    DZZbig(24,23) = 1.0_wp
    DZZbig(25,24) = 1.0_wp
    DZZbig(26,25) = 1.0_wp
    DZZbig(27,26) = 1.0_wp
    DZZbig(28,27) = 1.0_wp
    DZZbig(29,28) = 1.0_wp
    DZZbig(30,29) = 1.0_wp
                  
    DZZbig(35,30) = 1.0_wp
    DZZbig(36,31) = 1.0_wp
    DZZbig(37,32) = 1.0_wp
    DZZbig(38,33) = 1.0_wp
    DZZbig(39,34) = 1.0_wp
    DZZbig(40,35) = 1.0_wp
                  
    DZZbig(46,36) = 1.0_wp
    DZZbig(47,37) = 1.0_wp
    DZZbig(48,38) = 1.0_wp
    DZZbig(49,39) = 1.0_wp
    DZZbig(50,40) = 1.0_wp

    ! DDD
    DDDbig = 0.0_wp
    do i = 1, 10
       DDDbig(i,40+i) = 1.0_wp
    end do

    ! HH
    DHHbig = 0.0_wp
    DHHbig(1,51)  = exp(para(51));  
    DHHbig(12,52) = exp(para(52));  
    DHHbig(23,53) = exp(para(53));  
    DHHbig(34,54) = exp(para(54));  
    DHHbig(45,55) = exp(para(55));  
    DHHbig(56,56) = exp(para(56));  
    DHHbig(67,57) = exp(para(57));  
    DHHbig(78,58) = exp(para(58));  
    DHHbig(89,59) = exp(para(59));  
    DHHbig(100,60) = exp(para(60)); 

    DTT = DTTbig(:,psel)
    DRR = DRRbig(:,psel)
    DQQ = DQQbig(:,psel)
    DZZ = 0.0_wp
    DDD = DDDbig(:,psel)
    DZZ = DZZbig(:,psel)
    DHH = DHHbig(:,psel)

  end subroutine sysmat_diff

  function pmsv result (para)

    real(wp) :: para(60)

    para(1:60) = (/0.50_wp, 0.50_wp, 0.50_wp, 0.50_wp, 0.50_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.00_wp, 0.50_wp, 0.50_wp, 0.50_wp, 0.50_wp, 0.50_wp, 0.50_wp, 0.50_wp, 0.50_wp, 0.50_wp, 0.50_wp, -1.00_wp, -1.00_wp, -1.00_wp, -1.00_wp, -1.00_wp, -1.00_wp, -1.00_wp, -1.00_wp, -1.00_wp, -1.00_wp /)

  end function pmsv

end module generic_state_space
