module rbc
  use mkl95_precision, only: wp => dp

  use prior
  use filter
  use strid_walentin
  use gensys
  
  implicit none 

  character(len=*), parameter :: mname = 'rbc'

  character(len=*), parameter :: priorfile = 'models/rbc/prior.txt'
  character(len=*), parameter :: datafile = 'models/rbc/yh2.txt' !'/mq/home/m1eph00/octave-git/test-models/rbc-big/data/us1948-2009-y-h.asc'
  character(len=*), parameter :: transfile = 'models/rbc/trspec.txt'
  character(len=*), parameter :: initfile = ''
  character(len=*), parameter :: initwt = ''
  character(len=*), parameter :: varfile = 'models/rbc/rbc_diffuse.txt'
  integer, parameter ::  neq = 12, neta = 2, neps = 2, nobs = 80, npara = 12, ns = 12, ny = 2
  integer, parameter :: t0 = 0

  real(wp) :: alp, bet, del, gam, theta, rhoA, rhoG, sigA, sigG, rhoAG, rhoGA, corrAG

  real(wp), parameter :: REALLY_NEG = -10.0_wp**10

  ! data stuff 
  real(wp) :: YY(ny,nobs)

  ! prior stuff
  integer :: pshape(npara), pmask(npara)
  real(wp) :: pmean(npara), pstdd(npara), pfix(npara)

  integer :: pshape2(npara), pmask2(npara)
  real(wp) :: pmean2(npara), pstdd2(npara), pfix2(npara)
  ! prtr
  real(wp) :: trspec(4, npara)
contains

  include '../../base/helper_functions.f90'

  subroutine rbc_ss(ss, valid, para)

    real(wp), intent(in) :: para(npara)
    real(wp), intent(out):: ss(6)
    logical, intent(out) :: valid

    real(wp) :: rst, k_y, i_y, i_k, g_y, c_y, h_l

    ss = 0
    valid = .true.

    alp   = para(1)
    bet   = para(2)
    del   = para(3)
    gam   = para(4)
    theta = para(5)
    rhoA  = para(6)
    rhoG  = para(7)
    sigA  = para(8)
    sigG  = para(9)
    
    if (1.0_wp/bet-1.+del < 1e-7 .or. alp < 1e-7) then
       valid = .false.
       return
    end if
    
    rst = 1.0_wp/bet-1.0_wp+del
    k_y = alp/rst
    i_y = (1.0_wp-(1.0_wp-del))*k_y
    i_k = i_y/k_y
    g_y = 0.20_wp 
    c_y = 1.0_wp-i_y-g_y
    h_l = (1.0_wp-alp)/(theta*c_y)	! hours/(1-hours)
    
    ss = (/ c_y, i_y, g_y, i_k, h_l, rst /)

  end subroutine rbc_ss

  subroutine sysmat(p0, TT, RR, QQ, DD, ZZ, HH, info)
  
    real(wp), intent(in) :: p0(npara)

    integer :: eq_r, eq_w, eq_mu, eq_h, eq_eu, eq_mc, eq_y, eq_kacc, eq_Er, eq_Elam, eq_a, eq_g
    integer :: r_t, w_t, c_t, k_t, h_t, y_t, i_t, lam_t, Er_t, Elam_t, g_t, a_t
    integer :: Er_sh, Elam_sh, a_sh, g_sh

    real(wp) :: GAM0(neq, neq), GAM1(neq, neq), C(neq), PSI(neq, neps), PPI(neq, neta), CC(neq) 
    real(wp), intent(out) :: TT(neq, neq), RR(neq, neps), QQ(neps, neps), DD(ny), ZZ(ny, neq), HH(ny,ny)
    integer, intent(out) :: info
    real(wp) :: fmat, fwt, ywt, gev, loose
    integer :: eu(2), i
    real(wp) :: c_y, i_y, g_y, i_k, h_l, rst, ss(6)
    logical :: valid
    real(wp) :: DIV

    HH = 0.0_wp
    DD = 0.0_wp
    ZZ = 0.0_wp
    QQ = 0.0_wp

    info = 0
    DIV = 1.008_wp
    alp = p0(1)
    bet = p0(2)
    del = p0(3)
    gam = p0(4)
    theta = p0(5)
    rhoA = p0(6)
    rhoG = p0(7)
    sigA = p0(8)
    sigG = p0(9)
    rhoAG = p0(10)
    rhoGA = p0(11)
    corrAG = p0(10)

    call rbc_ss(ss, valid, p0)

    c_y = ss(1)
    i_y = ss(2)
    g_y = ss(3)
    i_k = ss(4)
    h_l = ss(5)
    rst = ss(6)

    !! equation indices !!
    eq_r    = 3  ! Firm FOC for r(t) !
    eq_w    = 4  ! Firm FOC for w(t) !
    eq_mu   = 5  ! Household Marginal Utility !
    eq_h    = 6  ! Household FOC for h(t) !
    eq_eu   = 7  ! Household Euler Equation !
    eq_mc   = 8  ! Market Clearing !
    eq_y    = 9  ! Production Function !
    eq_kacc = 10  ! Capital Accumulation !
    eq_Er   = 11 ! E(r(t+1)) !
    eq_Elam = 12	! E(lam(t+1)) ! 
    eq_a    = 1	! a process !
    eq_g    = 2	! b process !

    !! variable indices !!
    r_t    = 3
    w_t    = 4
    c_t    = 5
    k_t    = 6
    h_t    = 7
    y_t    = 8
    i_t    = 9
    lam_t  = 10
    Er_t   = 11
    Elam_t = 12
    g_t    = 2
    a_t    = 1

    !! expectation error indices (eta) !!
    Er_sh    = 1
    Elam_sh  = 2

    !! shock indices (eps) !!
    a_sh = 1
    g_sh = 2

    GAM0 = 0.0_wp
    GAM1 = 0.0_wp
    PSI = 0.0_wp
    PPI = 0.0_wp
    !!      1. Firm FOC for r(t)                             !!

    GAM0(eq_r,r_t)   =  1.0_wp
    GAM0(eq_r,y_t)   = -1.0_wp
    GAM1(eq_r,k_t)   = -1.0_wp


    !!      2. Firm FOC for w(t)                            !!

    GAM0(eq_w,w_t)   =  1.0_wp
    GAM0(eq_w,y_t)   = -1.0_wp
    GAM0(eq_w,h_t)   =  1.0_wp


    !!      3. HH FOC for Marginal Utility                  !!

    GAM0(eq_mu,lam_t)  = 1.0_wp
    GAM0(eq_mu,c_t  )  = gam
    GAM0(eq_mu,h_t  )  = (1.0_wp-gam)*theta*h_l


    !!      4. HH FOC for h(t)                              !!

    GAM0(eq_h,w_t)   = -1.0_wp
    GAM0(eq_h,c_t)   =  1.0_wp
    GAM0(eq_h,h_t)   =  h_l


    !!      5. HH Euler Equation                            !!

    GAM0(eq_eu,lam_t)  = -1.0_wp
    GAM0(eq_eu,Er_t )  =  rst/(rst+1.0_wp-del)
    GAM0(eq_eu,Elam_t) =  1.0_wp


    !!      5. Market Clearing Condition                    !!

    GAM0(eq_mc,c_t)   = -c_y
    GAM0(eq_mc,i_t)   = -i_y
    GAM0(eq_mc,g_t)   = -g_y
    GAM0(eq_mc,y_t)   =  1.0_wp


    !!      6. Production Function                          !!

    GAM0(eq_y,y_t)   =  1.0_wp
    GAM0(eq_y,h_t)   = -(1.0_wp-alp)
    GAM0(eq_y,a_t)   = -(1.0_wp-alp) 
    GAM1(eq_y,k_t)   =  alp


    !!      7. Capital Accumulation                         !!

    GAM0(eq_kacc,k_t)  =  1.0_wp
    GAM0(eq_kacc,i_t)  = -i_k
    GAM1(eq_kacc,k_t)  = (1.0_wp-del)


    !!      Expectation error                               !!

    !! E(r) !!
    GAM0(eq_Er,r_t)   = 1.0_wp
    GAM1(eq_Er,Er_t)  = 1.0_wp
    PPI(eq_Er,Er_sh)  = 1.0_wp

    !! E(lam) !!
    GAM0(eq_Elam,lam_t)   = 1.0_wp
    GAM1(eq_Elam,Elam_t)  = 1.0_wp
    PPI(eq_Elam,Elam_sh)  = 1.0_wp


    !!      Shock process                                   !!

    !! a(t) !!
    GAM0(eq_a,a_t) = 1.0_wp
    GAM1(eq_a,a_t) = rhoA
    GAM1(eq_a,g_t) = rhoAG
    PSI(eq_a,a_sh) = 1.0_wp

    !! g(t) !!
    GAM0(eq_g,g_t)  = 1.0_wp
    GAM1(eq_g,g_t)  = rhoG
    GAM1(eq_g,a_t)  = rhoAG
    PSI(eq_g,g_sh)  = 1.0_wp

    call do_gensys(TT, CC, RR, fmat, fwt, ywt, gev, eu, loose, GAM0, GAM1, C, PSI, PPI, DIV)

    info = eu(1)

    ! now get other matrices
    QQ(1,1) = sigA*sigA
    QQ(2,2) = sigG*sigG
    QQ(1,2) = corrAG*sigA*sigG
    QQ(2,1) = corrAG*sigA*sigG

    
    ZZ(1, y_t) = 1.0_wp
    ZZ(2, h_t) = 1.0_wp

  end subroutine sysmat

  function pmsv result (para)
    real(wp) :: para(npara)

    para = (/0.34_wp, 0.995_wp, 0.015_wp, 2.5_wp, 4.0_wp, 0.5_wp, 0.5_wp, 0.1_wp, 0.6_wp, 0.0_wp, 0.0_wp, 0.0_wp/)
  end function pmsv

  subroutine sysmat_diff(para, psel, neffpara, DTT, DRR, DQQ, DDD, DZZ, DHH)

    integer, intent(in) :: neffpara, psel(neffpara)
    real(wp), intent(in) :: para(npara)

    real(wp), intent(out) :: DTT(ns**2,neffpara), DRR(ns*neps,neffpara), DQQ(neps*neps, neffpara)
    real(wp), intent(out) :: DDD(ny,neffpara), DZZ(ns*ny,neffpara), DHH(ny*ny,neffpara)

    real(wp) :: TT(ns,ns), RR(ns,neps), QQ(neps,neps), DD(ny), ZZ(ny,ns), HH(ny,ny)
    real(wp) :: TT1(ns,ns), RR1(ns,neps), QQ1(neps,neps), DD1(ny), ZZ1(ny,ns), HH1(ny,ny)

    real(wp) :: dh(neffpara), p1(npara)
    real(wp), parameter :: ep = 6.0554544523833428e-6
    integer :: i,info

    call sysmat(para,TT,RR,QQ,DD,ZZ,HH,info)

!!$    if (inbounds(para) .ne. .true.) then
!!$       DTT = 0.0_wp
!!$       return
!!$    end if

    do i = 1,neffpara

       p1 = para
       dh(i) = ep*maxval((/abs(para(psel(i))),0.01_wp/))

       p1(psel(i)) = p1(psel(i)) + dh(i)
       call sysmat(p1,TT1,RR1,QQ1,DD1,ZZ1,HH1,info)

       call dcopy(ns*ns, TT1, 1, DTT(:,i), 1)
       call dcopy(ns*neps, RR1, 1, DRR(:,i), 1)
       call dcopy(neps*neps, QQ1, 1, DQQ(:,i), 1)
       call dcopy(ny, DD1, 1, DDD(:,i), 1)
       call dcopy(ns*ny, ZZ1, 1, DZZ(:,i), 1)
       call dcopy(ny*ny, HH1, 1, DHH(:,i), 1)

       call daxpy(ns*ns, -1.0_wp, TT, 1, DTT(:,i), 1)
       call daxpy(ns*neps, -1.0_wp, RR, 1, DRR(:,i), 1)
       call daxpy(neps*neps, -1.0_wp, QQ, 1, DQQ(:,i), 1)
       call daxpy(ny, -1.0_wp, DD, 1, DDD(:,i), 1)
       call daxpy(ns*ny, -1.0_wp, ZZ, 1, DZZ(:,i), 1)
       call daxpy(ny*ny, -1.0_wp, HH, 1, DHH(:,i), 1)

       call dscal(ns*ns, 1.0_wp/dh(i), DTT(:,i),1)
       call dscal(ns*neps, 1.0_wp/dh(i), DRR(:,i),1)
       call dscal(neps*neps, 1.0_wp/dh(i), DQQ(:,i),1)
       call dscal(ny, 1.0_wp/dh(i), DDD(:,i),1)
       call dscal(ns*ny, 1.0_wp/dh(i), DZZ(:,i),1)
       call dscal(ny*ny, 1.0_wp/dh(i), DHH(:,i),1)


    end do

  end subroutine sysmat_diff

end module rbc
