!!! sw.f90 --- 
!! 
!! Description: This module contains the Smets-Wouters model. 
!! 
!! Author: Ed Herbst [edward.p.herbst@frb.gov]
!! Last-Updated: 05/08/13
!! 
module sw

  use mkl95_precision, only: wp => dp

  use prior
  use filter
  use gensys

  implicit none 
  
  character(len=*), parameter :: mname = 'sw-new'

  character(len=*), parameter :: priorfile = '/mq/home/m1eph00/code/fortran/models/sw/prior.txt'!~_diffuse_rr.txt'!_uninformative.txt'
  character(len=*), parameter :: priorfile2 = '/mq/home/m1eph00/code/fortran/models/sw/prior.txt'!_uniformative.txt'
  character(len=*), parameter :: datafile = '/mq/home/m1eph00/code/fortran/models/sw/YY.txt'
  character(len=*), parameter :: transfile = '/mq/home/m1eph00/code/fortran/models/sw/trspec.txt'
  character(len=*), parameter :: initfile = '/mq/home/m1eph00/code/fortran/models/sw/diffuse_parasim.txt'
  character(len=*), parameter :: initwt = ''

  character(len=*), parameter :: varfile = '/mq/home/m1eph00/code/fortran/models/sw/smc_variance.txt'

  integer, parameter :: neq  = 50, neps = 7, neta = 12, npara = 41, ny = 7, ns = 50,nobs = 156
  integer, parameter :: t0 = 4

  real(wp), parameter :: REALLY_NEG = -10.0_wp**10

  ! data stuff 
  real(wp) :: YY(ny, nobs)

  ! prior stuff
  integer  :: pshape(npara), pmask(npara)
  real(wp) :: pmean(npara), pstdd(npara), pfix(npara)

  integer  :: pshape2(npara), pmask2(npara)
  real(wp) :: pmean2(npara), pstdd2(npara), pfix2(npara)

  ! prtr
  real(wp) :: trspec(4, npara)
  character(len=144) :: ns_str, fhGAM0, fhGAM1, outstr, fname

contains

  include '../../base/helper_functions.f90'
  subroutine swmodel(para, TT, RR, info)

    real(wp), intent(in) :: para(npara)

    integer, intent(out) :: info
    real(wp), intent(out) :: TT(neq, neq), RR(neq, neps) 

    real(wp) :: GAM0(neq, neq), GAM1(neq, neq), C(neq), PSI(neq, neps), PPI(neq, neta), CC(neq), G0(neq, neq), G1(neq, neq)

    ! parameters
    real(wp) :: varphi, sigmac, h, lamw, xiw, epsw, sigmal, del, xip, epsp, & 
         iotaw, iotap, ppsi, capphi, rpi, rho, ry, rdely, pibar, betin, lbar, &
         gambar, alp, g_y, rhoa, rhob, rhog, rhoi, rhor, rhop, rhow, mup, muw, &
         rhoga,  sigmaa, sigmab, sigmag, sigmai, sigmar, sigmap, sigmaw

    real(wp) :: gam, pistar, bet, rbar, rk_ss, w_ss, i_k, l_k, k_y, i_y, & 
         c_y, z_y, wl_c

    ! equation indices
    integer :: eq_euler, eq_inv, eq_q, eq_output, eq_caputl, eq_capsrv, & 
         eq_capev, eq_mkupp, eq_phlps, eq_caprnt, eq_mkupw, eq_wage, eq_mp, & 
         eq_res, eq_g, eq_b, eq_ia, eq_a, eq_lp, eq_lw, eq_mps, eq_lp1, eq_lw1, &
         eq_Ec, eq_Eq, eq_Ei, eq_Epi, eq_El, eq_Erk, eq_Ew, eq_euler_F, & 
         eq_inv_F, eq_q_F, eq_output_F, eq_caputl_F, eq_capsrv_F, eq_capev_F, &
         eq_mkupp_F, eq_caprnt_F, eq_mkupw_F, eq_res_F, eq_Ec_F, eq_Eq_F, & 
         eq_Ei_F, eq_El_F, eq_Erk_F, eq_ylag, eq_clag, eq_ilag, eq_wlag

    ! variable indices
    integer :: v_y, v_c, v_i, v_q, v_ks, v_k, v_z, v_rk, v_mup, v_pi, v_muw, & 
         v_w, v_l, v_r, v_g, v_b, v_ia, v_a, v_lp, v_lp1, v_lw, v_lw1, v_mp, &
         v_Ec, v_Eq, v_Ei, v_Epi, v_El, v_Erk, v_Ew, v_y_F, v_c_F, v_i_F, & 
         v_q_F, v_ks_F, v_k_F, v_z_F, v_rk_F, v_w_F, v_l_F, v_r_F, v_Ec_F, &
         v_Eq_F, v_Ei_F, v_El_F, v_Erk_F, v_ylag, v_clag, v_ilag, v_wlag
    
    ! shock and expectation error indices
    integer :: e_g, e_b, e_ia, e_a, e_lp, e_lw, e_mp, n_c, n_q, n_i, n_pi, & 
         n_l, n_rk, n_w, n_c_F, n_q_F, n_i_F, n_l_F, n_rk_F

    real(wp) :: fmat, fwt, ywt, gev, loose, DIV
    integer :: eu(2), i

    DIV = 1.01_wp

    !! assign names to parameters 
    varphi = para(1)        !    6.3324999999999996    6.3325  5.6560
    sigmac = para(2)        !    1.2312000000000001    1.2312  1.3512
    h      = para(3)        !    0.7205000000000000    0.7205  0.7201
    lamw   = para(4)        !    1.5000000000000000            1.5000
    xiw    = para(5)        !    0.7937000000000000    0.7937  0.4787
    epsw   = para(6)        !   10.0000000000000000            10.000 
    sigmal = para(7)        !    2.8401000000000001    2.8401  1.5121
    del    = para(8)        !    0.0250000000000000            0.0250
    xip    = para(9)        !    0.7813000000000000    0.7813  0.6410
    epsp   = para(10)       !   10.0000000000000000            10.000
    iotaw  = para(11)       !    0.4425000000000000    0.4425  0.6140
    iotap  = para(12)       !    0.3291000000000000    0.3291  0.2088
    ppsi   = para(13)       !    0.2648000000000000    0.2648  0.5946
    capphi = para(14)       !    1.4672000000000001    1.4672  1.6917
    rpi    = para(15)       !    1.7985000000000000    1.7985  2.0757
    rho    = para(16)       !    0.8258000000000000    0.8258  0.7888
    ry     = para(17)       !    0.0893000000000000    0.0893  0.0644
    rdely  = para(18)       !    0.2239000000000000    0.2239  0.2159
    pibar  = para(19)       !    0.7000000000000000    0.7000  0.6944
    betin  = para(20)       !    0.7420000000000000    0.7420  0.1420
    lbar   = para(21)       !    1.2918000000000001    1.2918  1.0320
    gambar = para(22)       !    0.3982000000000000    0.3982  0.4003
    alp    = para(23)       !    0.2400000000000000    0.2400  0.1779 
    g_y    = para(24)       !    0.1800000000000000            0.1800

    rhoa   = para(25)       !    0.9676000000000000    0.9768  0.9559  
    rhob   = para(26)       !    0.2703000000000000    0.2703  0.1546
    rhog   = para(27)       !    0.9930000000000000    0.9930  0.9870
    rhoi   = para(28)       !    0.5724000000000000    0.5724  0.7564
    rhor   = para(29)       !    0.3000000000000000    0.3000  0.1398
    rhop   = para(30)       !    0.8692000000000000    0.8692  0.6565
    rhow   = para(31)       !    0.9546000000000000    0.9546  0.9695

    mup    = para(32)       !    0.7652000000000000    0.7600  0.0
    muw    = para(33)       !    0.8935999999999999    0.8935  0.0 
    rhoga  = para(34)       !    0.0500000000000000    0.0500  0.4514  

    sigmaa = para(35)       !    0.4618000000000000    0.4618  0.5722
    sigmab = para(36)       !    0.1818000000000000    0.1818  0.4276
    sigmag = para(37)       !    0.6090000000000000    0.6090  0.2480
    sigmai = para(38)       !    0.4601000000000000    0.4601  0.1056
    sigmar = para(39)       !    0.2397000000000000    0.2397  0.1877
    sigmap = para(40)       !    0.1455000000000000    0.1455
    sigmaw = para(41)       !    0.2089000000000000    0.2089

    !! equation indices 
    eq_euler  = 8  !1   
    eq_inv    = 9  !2   
    eq_q      = 10  !3   
    eq_output = 11  !4   
    eq_caputl = 12  !5   
    eq_capsrv = 13  !6   
    eq_capev  = 14  !7   
    eq_mkupp  = 15  !8   
    eq_phlps  = 16  !9   
    eq_caprnt = 17  !10  
    eq_mkupw  = 18  !11  
    eq_wage   = 19  !12  
    eq_mp     = 20  !13  
    eq_res    = 21  !14  
    eq_g      = 1   !15  
    eq_b      = 2   !16  
    eq_ia     = 3   !17  
    eq_a      = 4   !18  
    eq_lp     = 5   !19  
    eq_lw     = 6   !20  
    eq_mps    = 7   !21  
    eq_lp1    = 22  
    eq_lw1    = 23  
    eq_Ec     = 24  
    eq_Eq     = 25  
    eq_Ei     = 26  
    eq_Epi    = 27  
    eq_El     = 28  
    eq_Erk    = 29  
    eq_Ew     = 30  
    eq_euler_F  = 31
    eq_inv_F    = 32
    eq_q_F      = 33
    eq_output_F = 34
    eq_caputl_F = 35
    eq_capsrv_F = 36
    eq_capev_F  = 37
    eq_mkupp_F  = 38
    eq_caprnt_F = 39
    eq_mkupw_F  = 40
    eq_res_F    = 41
    eq_Ec_F     = 42
    eq_Eq_F     = 43
    eq_Ei_F     = 44
    eq_El_F     = 45
    eq_Erk_F    = 46
    eq_ylag     = 47
    eq_clag     = 48
    eq_ilag     = 49
    eq_wlag     = 50

    !! variable indices 
    v_y       = 8 !1
    v_c       = 9 !2
    v_i       = 10 !3
    v_q       = 11 !4
    v_ks      = 12 !5
    v_k       = 13 !6
    v_z       = 14 !7
    v_rk      = 15 !8
    v_mup     = 16 !9
    v_pi      = 17 !10
    v_muw     = 18 !11
    v_w       = 19 !12
    v_l       = 20 !13
    v_r       = 21 !14
    v_g       = 1 !15
    v_b       = 2 !16
    v_ia      = 3 !17
    v_a       = 4 !18
    v_lp      = 5 !19
    v_lp1     = 22 
    v_lw      = 6 !21
    v_lw1     = 23
    v_mp      = 7 !23
    v_Ec      = 24
    v_Eq      = 25
    v_Ei      = 26
    v_Epi     = 27
    v_El      = 28
    v_Erk     = 29
    v_Ew      = 30
    v_y_F     = 31
    v_c_F     = 32
    v_i_F     = 33
    v_q_F     = 34
    v_ks_F    = 35
    v_k_F     = 36
    v_z_F     = 37
    v_rk_F    = 38
    v_w_F     = 39
    v_l_F     = 40
    v_r_F     = 41
    v_Ec_F    = 42
    v_Eq_F    = 43
    v_Ei_F    = 44
    v_El_F    = 45
    v_Erk_F   = 46
    v_ylag    = 47
    v_clag    = 48
    v_ilag    = 49
    v_wlag    = 50


    !! shock indices 
    e_g       = 1
    e_b       = 2
    e_ia      = 3
    e_a       = 4
    e_lp      = 5
    e_lw      = 6
    e_mp      = 7

    !! expectation errors 
    n_c       = 1
    n_q       = 2
    n_i       = 3
    n_pi      = 4
    n_l       = 5
    n_rk      = 6
    n_w       = 7
    n_c_F     = 8
    n_q_F     = 9
    n_i_F     = 10
    n_l_F     = 11
    n_rk_F    = 12


    !! steady state parameters 
    gam    = gambar/100.0_wp + 1.0_wp
    pistar = pibar /100.0_wp + 1.0_wp
    bet    = 1.0_wp/(betin/100.0_wp + 1.0_wp)
    rbar   = 100.0_wp*(bet**(-1)*gam**sigmac*pistar - 1.0_wp)

    rk_ss   = (gam**sigmac)/bet - (1.0_wp-del)
    w_ss    = (alp**alp*(1.0_wp-alp)**(1.0_wp-alp)/(capphi*rk_ss**alp))**(1.0_wp/(1.0_wp-alp))

    !! great (and not so great) ratios 
    i_k      = (1.0_wp - (1.0_wp-del)/gam)*gam
    l_k      = (1.0_wp - alp)/alp * rk_ss/w_ss
    k_y      = capphi*l_k**(alp-1.0_wp)
    i_y      = (gam-1.0_wp+del)*k_y
    c_y      = 1.0_wp - g_y - i_y
    z_y      = rk_ss*k_y
    wl_c     = 1.0_wp/lamw*(1.0_wp-alp)/alp*rk_ss*k_y/c_y

    GAM0 = 0.0_wp
    GAM1 = 0.0_wp
    PSI  = 0.0_wp
    PPI  = 0.0_wp
    C    = 0.0_wp


    !!      1. Consumption Euler Equation               !!

    !! sticky prices and wages 
    GAM0(eq_euler,v_c  ) =  1.0_wp
    GAM0(eq_euler,v_Ec ) = -1.0_wp/(1.0_wp+h/gam)
    GAM0(eq_euler,v_l  ) = -(sigmac-1.0_wp)*wl_c/(sigmac*(1.0_wp+h/gam))
    GAM0(eq_euler,v_El ) =  (sigmac-1.0_wp)*wl_c/(sigmac*(1.0_wp+h/gam))
    GAM0(eq_euler,v_r  ) =  (1.0_wp-h/gam)/( (1.0_wp+h/gam)*sigmac)
    GAM0(eq_euler,v_Epi) = -(1.0_wp-h/gam)/( (1.0_wp+h/gam)*sigmac)
    GAM0(eq_euler,v_b  ) = -1.0_wp !(1.0_wp-h/gam)/( (1.0_wp+h/gam)*sigmac)
    GAM1(eq_euler,v_c  ) =  (h/gam)/(1.0_wp+h/gam)

    !! flexible prices and wages
    GAM0(eq_euler_F,v_c_F ) =  1.0_wp
    GAM0(eq_euler_F,v_Ec_F) = -1.0_wp/(1.0_wp+h/gam)
    GAM0(eq_euler_F,v_l_F ) = -(sigmac-1.0_wp)*wl_c/(sigmac*(1.0_wp+h/gam))
    GAM0(eq_euler_F,v_El_F) =  (sigmac-1.0_wp)*wl_c/(sigmac*(1.0_wp+h/gam))
    GAM0(eq_euler_F,v_r_F ) =  (1.0_wp-h/gam)/( (1.0_wp+h/gam)*sigmac)
    GAM0(eq_euler_F,v_b   ) = -1.0_wp !(1.0_wp-h/gam)/( (1.0_wp+h/gam)*sigmac)
    GAM1(eq_euler_F,v_c_F ) =  (h/gam)/(1.0_wp+h/gam)


    !!      2. Investment Euler Equation               !!


    !! sticky prices and wages !!
    GAM0(eq_inv,v_i )  =  1.0_wp
    GAM0(eq_inv,v_Ei)  = -bet*gam**(1.0_wp-sigmac)/(1.0_wp+bet*gam**(1.0_wp-sigmac))
    GAM0(eq_inv,v_q )  = -1.0_wp/((gam**2)*varphi)*1.0_wp/(1.0_wp+bet*gam**(1.0_wp-sigmac))
    GAM0(eq_inv,v_ia)  = -1.0_wp
    GAM1(eq_inv,v_i )  =  1.0_wp/(1.0_wp+bet*gam**(1.0_wp-sigmac))

    !! flexible prices and wages !!
    GAM0(eq_inv_F,v_i_F )  =  1.0_wp
    GAM0(eq_inv_F,v_Ei_F)  = -bet*gam**(1.0_wp-sigmac)/(1.0_wp+bet*gam**(1.0_wp-sigmac))
    GAM0(eq_inv_F,v_q_F )  = -1.0_wp/((gam**2)*varphi)*1.0_wp/(1.0_wp+bet*gam**(1.0_wp-sigmac))
    GAM0(eq_inv_F,v_ia  )  = -1.0_wp
    GAM1(eq_inv_F,v_i_F )  =  1.0_wp/(1.0_wp+bet*gam**(1.0_wp-sigmac))


    !!      3. Value of Capital                        !!

    !! sticky prices and wages !!
    GAM0(eq_q,v_q  ) =  1.0_wp
    GAM0(eq_q,v_Eq ) = -bet*gam**(-sigmac)*(1.0_wp-del)
    GAM0(eq_q,v_Erk) = -(1.0_wp-bet*gam**(-sigmac)*(1.0_wp-del))
    GAM0(eq_q,v_r  ) =  1.0_wp
    GAM0(eq_q,v_Epi) = -1.0_wp
    GAM0(eq_q,v_b  ) = -1.0_wp/((1.0_wp-h/gam)/((1.0_wp+h/gam)*sigmac))

    !! flexible prices and wages !!
    GAM0(eq_q_F,v_q_F  ) =  1.0_wp
    GAM0(eq_q_F,v_Eq_F ) = -bet*gam**(-sigmac)*(1.0_wp-del)
    GAM0(eq_q_F,v_Erk_F) = -(1.0_wp-bet*gam**(-sigmac)*(1.0_wp-del))
    GAM0(eq_q_F,v_r_F  ) =  1.0_wp
    GAM0(eq_q_F,v_b    ) = -1.0_wp/((1.0_wp-h/gam)/((1.0_wp+h/gam)*sigmac))


    !!      4. Aggregate Production Function          !!

    !! sticky prices and wages !!
    GAM0(eq_output,v_y ) =  1.0_wp
    GAM0(eq_output,v_ks) = -capphi*alp
    GAM0(eq_output,v_l ) = -capphi*(1.0_wp-alp)
    GAM0(eq_output,v_a ) = -capphi

    !! flexible prices and wages !!
    GAM0(eq_output_F,v_y_F ) =  1.0_wp
    GAM0(eq_output_F,v_ks_F) = -capphi*alp
    GAM0(eq_output_F,v_l_F ) = -capphi*(1.0_wp-alp)
    GAM0(eq_output_F,v_a   ) = -capphi


    !!      5. Capital Utilization                   !!

    !! sticky prices and wages !!
    GAM0(eq_caputl,v_ks) =  1.0_wp
    GAM1(eq_caputl,v_k ) =  1.0_wp
    GAM0(eq_caputl,v_z ) = -1.0_wp

    !! flexible prices and wages !!
    GAM0(eq_caputl_F,v_ks_F) =  1.0_wp
    GAM1(eq_caputl_F,v_k_F ) =  1.0_wp
    GAM0(eq_caputl_F,v_z_F ) = -1.0_wp


    !!      6. Rental Rate of Capital               !!

    !! sticky prices and wages !!
    GAM0(eq_capsrv,v_z ) =  1.0_wp
    GAM0(eq_capsrv,v_rk) = -(1.0_wp-ppsi)/ppsi

    !! flexible prices and wages !!
    GAM0(eq_capsrv_F,v_z_F ) =  1.0_wp
    GAM0(eq_capsrv_F,v_rk_F) = -(1.0_wp-ppsi)/ppsi


    !!      7. Evolution of Capital                 !!

    !! changed v_ia to (1.0_wp-(1.0_wp-del)/gam)*(1.0_wp+bet*gam**(1.0_wp-sigmac))*gam**2*varphi !!

    !! sticky prices and wages !!
    GAM0(eq_capev,v_k ) =  1.0_wp
    GAM0(eq_capev,v_ia) = -(1.0_wp-(1.0_wp-del)/gam)*gam**2*varphi*(1.0_wp+bet*gam**(1-sigmac))
    GAM0(eq_capev,v_i ) = -(1.0_wp-(1.0_wp-del)/gam)
    GAM1(eq_capev,v_k ) =  (1.0_wp-del)/gam

    !! flexible prices and wages !!
    GAM0(eq_capev_F,v_k_F ) =  1.0_wp
    GAM0(eq_capev_F,v_ia  ) = -(1.0_wp-(1.0_wp-del)/gam)*gam**2*varphi*(1.0_wp+bet*gam**(1-sigmac))
    GAM0(eq_capev_F,v_i_F ) = -(1.0_wp-(1.0_wp-del)/gam)
    GAM1(eq_capev_F,v_k_F ) =  (1.0_wp-del)/gam


    !!      8. Price Markup                       !!

    !! sticky prices and wages !! 
    GAM0(eq_mkupp,v_mup) =  1.0_wp
    GAM0(eq_mkupp,v_ks ) = -alp
    GAM0(eq_mkupp,v_l  ) =  alp
    GAM0(eq_mkupp,v_a  ) = -1.0_wp
    GAM0(eq_mkupp,v_w  ) =  1.0_wp

    !! flexible prices and wages !!
    GAM0(eq_mkupp_F,v_ks_F ) = -alp
    GAM0(eq_mkupp_F,v_l_F  ) =  alp
    GAM0(eq_mkupp_F,v_a    ) = -1.0_wp
    GAM0(eq_mkupp_F,v_w_F  ) =  1.0_wp


    !!      9. Phillips Curve                     !!

    !! sticky prices and wages !!
    GAM0(eq_phlps,v_pi ) =  1.0_wp
    GAM0(eq_phlps,v_Epi) = -bet*gam**(1.0_wp-sigmac)/(1.0_wp+bet*gam**(1.0_wp-sigmac)*iotap)
    GAM0(eq_phlps,v_mup) =  (1.0_wp-xip)/xip*1.0_wp/((capphi-1.0_wp)*epsp+1.0_wp)*1.0_wp/(1.0_wp+bet*gam**(1.0_wp-sigmac)*iotap)*(1.0_wp-bet*gam**(1.0_wp-sigmac)*xip)
    GAM0(eq_phlps,v_lp ) = -1.0_wp
    GAM1(eq_phlps,v_pi ) = iotap/(1.0_wp+bet*gam**(1.0_wp-sigmac)*iotap)

    !! flexible prices and wages !!
    !! not necesary !!


    !!     10. Rental Rate of Capital            !!

    !! sticky prices and wages !!
    GAM0(eq_caprnt,v_rk) =  1.0_wp
    GAM0(eq_caprnt,v_ks) =  1.0_wp	!was k
    GAM0(eq_caprnt,v_l ) = -1.0_wp
    GAM0(eq_caprnt,v_w ) = -1.0_wp

    !! flexible prices and wages !!
    GAM0(eq_caprnt_F,v_rk_F) =  1.0_wp
    GAM0(eq_caprnt_F,v_ks_F) =  1.0_wp !was k@
    GAM0(eq_caprnt_F,v_l_F ) = -1.0_wp
    GAM0(eq_caprnt_F,v_w_F ) = -1.0_wp


    !!     11. Wage Markup                       !!

    !! sticky prices and wages !!
    GAM0(eq_mkupw,v_muw) =  1.0_wp
    GAM0(eq_mkupw,v_w  ) = -1.0_wp
    GAM0(eq_mkupw,v_l  ) =  sigmal
    GAM0(eq_mkupw,v_c  ) =  1.0_wp/(1.0_wp-h/gam)
    GAM1(eq_mkupw,v_c  ) =  (h/gam)/(1.0_wp-h/gam)

    !! flexible prices and wages !!
    GAM0(eq_mkupw_F,v_w_F  ) = -1.0_wp
    GAM0(eq_mkupw_F,v_l_F  ) =  sigmal
    GAM0(eq_mkupw_F,v_c_F  ) =  1.0_wp/(1.0_wp-h/gam)
    GAM1(eq_mkupw_F,v_c_F  ) =  (h/gam)/(1.0_wp-h/gam)


    !!     12. Evolution of Wages               !!


    !! sticky prices and wages !!
    GAM0(eq_wage,v_w  ) =  1.0_wp
    GAM0(eq_wage,v_Ew ) = -(1.0_wp-1.0_wp/(1.0_wp+bet*gam**(1.0_wp-sigmac)))
    GAM0(eq_wage,v_Epi) = -(1.0_wp-1.0_wp/(1.0_wp+bet*gam**(1.0_wp-sigmac)))
    GAM0(eq_wage,v_pi ) =  (1.0_wp+bet*gam**(1.0_wp-sigmac)*iotaw)/(1.0_wp+bet*gam**(1.0_wp-sigmac))
    GAM0(eq_wage,v_muw) =  1.0_wp/(1.0_wp+bet*gam**(1.0_wp-sigmac))*(1.0_wp-bet*gam**(1.0_wp-sigmac)*xiw)*(1.0_wp-xiw)/(xiw*((lamw-1.0_wp)*epsw+1.0_wp))
    GAM0(eq_wage,v_lw ) = -1.0_wp
    GAM1(eq_wage,v_w  ) =  1.0_wp/(1.0_wp+bet*gam**(1.0_wp-sigmac))
    GAM1(eq_wage,v_pi ) =  iotaw/(1.0_wp+bet*gam**(1.0_wp-sigmac))

    !! flexible prices and wages !!
    !! not necessary !!


    !!    13. Monetary Policy Rule               !!


    !! sticky prices and wages !!
    GAM0(eq_mp,v_r  )   =  1.0_wp
    GAM1(eq_mp,v_r  )   =  rho
    GAM0(eq_mp,v_pi )   = -(1.0_wp-rho)*rpi
    GAM0(eq_mp,v_y  )   = -(1.0_wp-rho)*ry-rdely
    GAM1(eq_mp,v_y  )   = -rdely
    GAM0(eq_mp,v_y_F)   =  (1.0_wp-rho)*ry+rdely
    GAM1(eq_mp,v_y_F)   =  rdely
    GAM0(eq_mp,v_mp )   = -1.0_wp

    !! flexible prices and wages !!
    !! not necessary !!


    !!   1.0_wp4. Resource Constraint               !!


    !! sticky prices and wages !!
    GAM0(eq_res,v_y) =  1.0_wp
    GAM0(eq_res,v_c) = -c_y
    GAM0(eq_res,v_i) = -i_y
    GAM0(eq_res,v_z) = -z_y
    GAM0(eq_res,v_g) = -1.0_wp

    !! flexible prices and wages !!
    GAM0(eq_res_F,v_y_F) =  1.0_wp
    GAM0(eq_res_F,v_c_F) = -c_y
    GAM0(eq_res_F,v_i_F) = -i_y
    GAM0(eq_res_F,v_z_F) = -z_y
    GAM0(eq_res_F,v_g  ) = -1.0_wp


    !!   Exogenous Processes                     !!


    !! government spending !!
    GAM0(eq_g,v_g) =  1.0_wp
    GAM1(eq_g,v_g) =  rhog
    PSI(eq_g,e_g)  =  1.0_wp
    PSI(eq_g,e_a)  =  rhoga

    !! asset shock !!
    GAM0(eq_b,v_b) =  1.0_wp
    GAM1(eq_b,v_b) =  rhob
    PSI(eq_b,e_b)  =  1.0_wp

    !! investment specific tech change !!
    GAM0(eq_ia,v_ia) = 1.0_wp
    GAM1(eq_ia,v_ia) = rhoi
    PSI(eq_ia,e_ia)  = 1.0_wp

    !! neutral technology !!
    GAM0(eq_a,v_a) = 1.0_wp
    GAM1(eq_a,v_a) = rhoa
    PSI(eq_a,e_a) = 1.0_wp

    !! price mark-up shock !!
    GAM0(eq_lp,v_lp)  =  1.0_wp
    GAM1(eq_lp,v_lp)  =  rhop
    GAM1(eq_lp,v_lp1) = -mup
    PSI(eq_lp,e_lp)  =  1.0_wp

    GAM0(eq_lp1,v_lp1) = 1.0_wp
    PSI(eq_lp1,e_lp ) = 1.0_wp

    !! wage mark-up shock !!
    GAM0(eq_lw,v_lw)  =  1.0_wp
    GAM1(eq_lw,v_lw)  =  rhow
    GAM1(eq_lw,v_lw1) = -muw
    PSI(eq_lw,e_lw)   =  1.0_wp

    GAM0(eq_lw1,v_lw1) = 1.0_wp
    PSI(eq_lw1,e_lw ) = 1.0_wp

    !! monetary policy shock !!
    GAM0(eq_mps,v_mp) = 1.0_wp
    GAM1(eq_mps,v_mp) = rhor
    PSI(eq_mps,e_mp) = 1.0_wp


    !!    Rational Expectations Errors       !!

    !! E(c) !!
    !! sticky prices and wages !!
    GAM0(eq_Ec,v_c ) = 1.0_wp
    GAM1(eq_Ec,v_Ec) = 1.0_wp
    PPI(eq_Ec,n_c ) = 1.0_wp

    !! flexible prices and wages !!
    GAM0(eq_Ec_F,v_c_F ) = 1.0_wp
    GAM1(eq_Ec_F,v_Ec_F) = 1.0_wp
    PPI(eq_Ec_F,n_c_F ) = 1.0_wp

    !! E(q) !!
    !! sticky prices and wages !!
    GAM0(eq_Eq,v_q ) = 1.0_wp
    GAM1(eq_Eq,v_Eq) = 1.0_wp
    PPI(eq_Eq,n_q ) = 1.0_wp

    !! flexible prices and wages !!
    GAM0(eq_Eq_F,v_q_F ) = 1.0_wp
    GAM1(eq_Eq_F,v_Eq_F) = 1.0_wp
    PPI(eq_Eq_F,n_q_F) = 1.0_wp

    !! E(i) !!
    !! sticky prices and wages !!
    GAM0(eq_Ei,v_i ) = 1.0_wp
    GAM1(eq_Ei,v_Ei) = 1.0_wp
    PPI(eq_Ei,n_i ) = 1.0_wp

    !! flexible prices and wages !!
    GAM0(eq_Ei_F,v_i_F ) = 1.0_wp
    GAM1(eq_Ei_F,v_Ei_F) = 1.0_wp
    PPI(eq_Ei_F,n_i_F ) = 1.0_wp

    !! E(pi) !!
    !! sticky prices and wages !!
    GAM0(eq_Epi,v_pi ) = 1.0_wp
    GAM1(eq_Epi,v_Epi) = 1.0_wp
    PPI(eq_Epi,n_pi ) = 1.0_wp

    !! E(l) !!
    !! sticky prices and wages !!
    GAM0(eq_El,v_l ) = 1.0_wp
    GAM1(eq_El,v_El) = 1.0_wp
    PPI(eq_El,n_l ) = 1.0_wp

    !! flexible prices and wages !!
    GAM0(eq_El_F,v_l_F ) = 1.0_wp
    GAM1(eq_El_F,v_El_F) = 1.0_wp
    PPI(eq_El_F,n_l_F ) = 1.0_wp

    !! E(rk) !!
    !! sticky prices and wages !!
    GAM0(eq_Erk,v_rk ) = 1.0_wp
    GAM1(eq_Erk,v_Erk) = 1.0_wp
    PPI(eq_Erk,n_rk ) = 1.0_wp

    !! flexible prices and wages !!
    GAM0(eq_Erk_F,v_rk_F ) = 1.0_wp
    GAM1(eq_Erk_F,v_Erk_F) = 1.0_wp
    PPI(eq_Erk_F,n_rk_F ) = 1.0_wp

    !! E(w) !!
    !! sticky prices and wages !!
    GAM0(eq_Ew,v_w ) = 1.0_wp
    GAM1(eq_Ew,v_Ew) = 1.0_wp
    PPI(eq_Ew,n_w ) = 1.0_wp



    !!    Augmentation                       !!

    !! y !!
    GAM0(eq_ylag,v_ylag) = 1.0_wp
    GAM1(eq_ylag,v_y)    = 1.0_wp

    !! c !!
    GAM0(eq_clag,v_clag) = 1.0_wp
    GAM1(eq_clag,v_c)    = 1.0_wp

    !! i !!
    GAM0(eq_ilag,v_ilag) = 1.0_wp
    GAM1(eq_ilag,v_i)    = 1.0_wp

    !! w !!
    GAM0(eq_wlag,v_wlag) = 1.0_wp
    GAM1(eq_wlag,v_w)    = 1.0_wp

    !!      QZ(generalized Schur) decomposition by GENSYS
    call do_gensys(TT, CC, RR, fmat, fwt, ywt, gev, eu, loose, & 
         GAM0, GAM1, C, PSI, PPI, DIV)

    info = eu(1)*eu(2)

  end subroutine swmodel

  subroutine sysmat(para, TT, RR, QQ, DD, ZZ, HH, info)

    integer, intent(out) :: info

    real(wp), intent(in) :: para(npara)

    real(wp), intent(out) :: TT(neq, neq), RR(neq, neps), QQ(neps, neps), &
         DD(ny), ZZ(ny, neq), HH(ny, ny)

    ! parameters
    real(wp) :: gambar, lbar, pibar, sigmac, betin, gam, bet, pistar, rbar, &
         sigmaa, sigmab, sigmag, sigmai, sigmar, sigmap, sigmaw

    ! observation, variable, and shock indices
    integer :: eq_ygr, eq_cgr, eq_igr, eq_wgr, eq_lnh, eq_pi, eq_r, &
         v_y, v_c, v_i, v_w, v_l, v_pi, v_r, v_ylag, v_clag, v_ilag, v_wlag, &
         e_g, e_b, e_ia, e_a, e_lp, e_lw, e_mp

    gambar    = para(22);
    lbar      = para(21);
    pibar     = para(19);
    sigmac    = para(2);
    betin     = para(20);
    gam       = gambar/100 + 1;
    bet       = 1/(betin/100 + 1);
    pistar    = pibar/100 + 1;
    rbar      = 100*(bet**(-1)*gam**sigmac*pistar - 1);

    sigmaa = para(35);
    sigmab = para(36);
    sigmag = para(37);
    sigmai = para(38);
    sigmar = para(39);
    sigmap = para(40);
    sigmaw = para(41);


    !! observation indices 
    eq_ygr = 1;
    eq_cgr = 2;
    eq_igr = 3;
    eq_wgr = 4;
    eq_lnh = 5;
    eq_pi  = 6;
    eq_r   = 7;

    !! variable indices 
    v_y    = 8 !1;
    v_c    = 9 !2;
    v_i    = 10 !3;
    v_w    = 19 !12;
    v_l    = 20 !13;
    v_pi   = 17 !10;
    v_r    = 21 !14;
    v_ylag = 47;
    v_clag = 48;
    v_ilag = 49;
    v_wlag = 50;

    !! shock indices 
    e_g       = 1;
    e_b       = 2;
    e_ia      = 3;
    e_a       = 4;
    e_lp      = 5;
    e_lw      = 6;
    e_mp      = 7;


    !! transition equations with augmentation       !!
    !! z(t) = T1*z(t-1) + T0*e(t)                   !!
    !! x(t) = [z(t)',...]'                          !!
    !!                                              !!
    !! x(t) = TT*x(t-1) + RR*e(t)                   !! 
    !! e(t) ~ iid N(0,QQ)                           !! 

    call swmodel(para, TT, RR, info)

    QQ = 0.0_wp
    QQ(e_g,e_g  ) = sigmag**2.0_wp;
    QQ(e_b,e_b  ) = sigmab**2.0_wp;
    QQ(e_ia,e_ia) = sigmai**2.0_wp;
    QQ(e_a,e_a  ) = sigmaa**2.0_wp;
    QQ(e_lp,e_lp) = sigmap**2.0_wp;
    QQ(e_lw,e_lw) = sigmaw**2.0_wp;
    QQ(e_mp,e_mp) = sigmar**2.0_wp;

    !! measurement equations                        !!
    !!                                              !!
    !! dlGDP  = gambar + y - y(-1)                  !!
    !! dlCONS = gambar + c - c(-1)                  !!
    !! dlINV  = gambar + i - i(-1)                  !!
    !! dlWAGE = gambar + w - w(-1)                  !!
    !! dlHOUR = lbar   + l                          !!
    !! INF    = pibar  + pi                         !!
    !! FEDFND = rbar   + r                          !!
    ! DD
    DD = 0.0_wp
    DD(eq_ygr) = gambar;
    DD(eq_cgr) = gambar;
    DD(eq_igr) = gambar;
    DD(eq_wgr) = gambar;
    DD(eq_lnh) = lbar;
    DD(eq_pi) = pibar;
    DD(eq_r) = rbar;

    ! ZZ
    ZZ = 0.0_wp
    ZZ(eq_ygr,v_y   ) =  1;
    ZZ(eq_ygr,v_ylag) = -1;
    ZZ(eq_cgr,v_c   ) =  1;
    ZZ(eq_cgr,v_clag) = -1;
    ZZ(eq_igr,v_i   ) =  1;
    ZZ(eq_igr,v_ilag) = -1;
    ZZ(eq_wgr,v_w   ) =  1;
    ZZ(eq_wgr,v_wlag) = -1;
    ZZ(eq_lnh,v_l   ) =  1;
    ZZ(eq_pi,v_pi   ) =  1;
    ZZ(eq_r,v_r     ) =  1;

    HH = 0.0_wp

  end subroutine sysmat

  function pmsv result (para)
    real(wp) :: para(npara)

    para = (/6.3325_wp, 1.2312_wp, 0.7205_wp, 1.5000_wp, 0.7937_wp, 10.000_wp, 2.8401_wp, 0.0250_wp, 0.7813_wp, 10.000_wp, 0.4425_wp, 0.3291_wp, 0.2648_wp, 1.4672_wp, 1.7985_wp, 0.8258_wp, 0.0893_wp, 0.2239_wp, 0.7000_wp, 0.7420_wp, 1.2918_wp, 0.3982_wp, 0.2400_wp, 0.1800_wp, 0.9676_wp, 0.2703_wp, 0.9930_wp, 0.5724_wp, 0.3000_wp, 0.8692_wp, 0.9546_wp, 0.7652_wp, 0.8936_wp, 0.0500_wp, 0.4618_wp, 0.1818_wp, 0.6090_wp, 0.4601_wp, 0.2397_wp, 0.1455_wp, 0.2089_wp/)
  end function pmsv

  subroutine sysmat_diff(para, psel, neffpara, DTT, DRR, DQQ, DDD, DZZ, DHH)

    integer, intent(in) :: neffpara, psel(neffpara)
    real(wp), intent(in) :: para(npara)

    real(wp), intent(out) :: DTT(ns**2,neffpara), DRR(ns*neps,neffpara), DQQ(neps*neps, neffpara)
    real(wp), intent(out) :: DDD(ny,neffpara), DZZ(ns*ny,neffpara), DHH(ny*ny,neffpara)

    real(wp) :: TT(ns,ns), RR(ns,neps), QQ(neps,neps), DD(ny), ZZ(ny,ns), HH(ny,ny)
    real(wp) :: TT1(ns,ns), RR1(ns,neps), QQ1(neps,neps), DD1(ny), ZZ1(ny,ns), HH1(ny,ny)

    real(wp) :: dh(neffpara), p1(npara)
    real(wp), parameter :: ep = 6.0554544523833428e-6
    integer :: i,info

    call sysmat(para,TT,RR,QQ,DD,ZZ,HH,info)
    
!!$    if (inbounds(para) .ne. .true.) then
!!$       DTT = 0.0_wp
!!$       return
!!$    end if

    do i = 1,neffpara

       p1 = para
       dh(i) = ep*maxval((/abs(para(psel(i))),0.01_wp/))

       p1(psel(i)) = p1(psel(i)) + dh(i)
       call sysmat(p1,TT1,RR1,QQ1,DD1,ZZ1,HH1,info)

       call dcopy(ns*ns, TT1, 1, DTT(:,i), 1)
       call dcopy(ns*neps, RR1, 1, DRR(:,i), 1)
       call dcopy(neps*neps, QQ1, 1, DQQ(:,i), 1)
       call dcopy(ny, DD1, 1, DDD(:,i), 1)
       call dcopy(ns*ny, ZZ1, 1, DZZ(:,i), 1)
       call dcopy(ny*ny, HH1, 1, DHH(:,i), 1)
       
       call daxpy(ns*ns, -1.0_wp, TT, 1, DTT(:,i), 1)
       call daxpy(ns*neps, -1.0_wp, RR, 1, DRR(:,i), 1)
       call daxpy(neps*neps, -1.0_wp, QQ, 1, DQQ(:,i), 1)
       call daxpy(ny, -1.0_wp, DD, 1, DDD(:,i), 1)
       call daxpy(ns*ny, -1.0_wp, ZZ, 1, DZZ(:,i), 1)
       call daxpy(ny*ny, -1.0_wp, HH, 1, DHH(:,i), 1)
       
       call dscal(ns*ns, 1.0_wp/dh(i), DTT(:,i),1)
       call dscal(ns*neps, 1.0_wp/dh(i), DRR(:,i),1)
       call dscal(neps*neps, 1.0_wp/dh(i), DQQ(:,i),1)
       call dscal(ny, 1.0_wp/dh(i), DDD(:,i),1)
       call dscal(ns*ny, 1.0_wp/dh(i), DZZ(:,i),1)
       call dscal(ny*ny, 1.0_wp/dh(i), DHH(:,i),1)
       
       
    end do
    
  end subroutine sysmat_diff
end module sw

